unit HRoutines;

interface

uses
  Windows, SysUtils, Forms,
  D3DX9, DXFile;

type
  //-----------------------------------------------------------------------------
  // Name: TD3DXFrameDerived
  // Desc: Structure derived from TD3DXFrame so we can add some app-specific
  //       info that will be stored with each frame
  //-----------------------------------------------------------------------------
  PD3DXFrameDerived = ^TD3DXFrameDerived;
  TD3DXFrameDerived = packed record { public TD3DXFrame - waiting for Delphi8 }
    Name:               PAnsiChar;
    TransformationMatrix: TD3DXMatrix;

    pMeshContainer:     PD3DXMeshContainer;

    pFrameSibling:      PD3DXFrame;
    pFrameFirstChild:   PD3DXFrame;
    ////////////////////////////////////////////
    UserData: DWORD;
  end;


//-----------------------------------------------------------------------------
// Name: class CAllocateHierarchy
// Desc: Custom version of ID3DXAllocateHierarchy with custom methods to create
//       frames and meshcontainers.
//-----------------------------------------------------------------------------
type
  TAllocateHierarchy = class(ID3DXAllocateHierarchy)
  public
    function CreateFrame(Name: PAnsiChar; out ppNewFrame: PD3DXFrame): HResult; override;
    function CreateMeshContainer(Name: PAnsiChar; const pMeshData: TD3DXMeshData;
        pMaterials: PD3DXMaterial; pEffectInstances: PD3DXEffectInstance;
        NumMaterials: DWORD; pAdjacency: PDWORD; pSkinInfo: ID3DXSkinInfo;
        out ppNewMeshContainer: PD3DXMeshContainer): HResult; override;
    function DestroyFrame(pFrameToFree: PD3DXFrame): HResult; override;
    function DestroyMeshContainer(pMeshContainerToFree: PD3DXMeshContainer): HResult; override;

    constructor Create;
  end;

  TD3DXLoadUserData = class(ID3DXLoadUserData)
    function LoadTopLevelData(pXofChildData: IDirectXFileData): HResult; override;

    function LoadFrameChildData(pFrame: PD3DXFrame;
        pXofChildData: IDirectXFileData): HResult; override;

    function LoadMeshChildData(pMeshContainer: PD3DXMeshContainer;
        pXofChildData: IDirectXFileData): HResult; override;
  end;

  TD3DXSaveUserData = class(ID3DXSaveUserData)
    function AddFrameChildData(pFrame: PD3DXFrame;
        pXofSave: IDirectXFileSaveObject; pXofFrameData: IDirectXFileData): HResult; override;

    function AddMeshChildData(pMeshContainer: PD3DXMeshContainer;
        pXofSave: IDirectXFileSaveObject; pXofMeshData: IDirectXFileData): HResult; override;

    // NOTE: this is called once per Save.  All top level objects should be added using the
    //    provided interface.  One call adds objects before the frame hierarchy, the other after
    function AddTopLevelDataObjectsPre(pXofSave: IDirectXFileSaveObject): HResult; override;
    function AddTopLevelDataObjectsPost(pXofSave: IDirectXFileSaveObject): HResult; override;

    // callbacks for the user to register and then save templates to the XFile
    function RegisterTemplates(pXFileApi: IDirectXFile): HResult; override;
    function SaveTemplates(pXofSave: IDirectXFileSaveObject): HResult; override;
  public
    Counter, Counter2: Integer;
  end;

var
  RootFrame: PD3DXFrame = nil; // TAllocateHierarchy RootFrame
  AutoGenerateUserData: Boolean = True;

implementation

uses Classes, MeshHierarchyUnit;

var
  LoadGlobalPre: Boolean = True;

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
{ CAllocateHierarchy }

constructor TAllocateHierarchy.Create;
begin
  inherited Create;
  LoadGlobalPre:= True;
end;

function TAllocateHierarchy.CreateFrame(Name: PAnsiChar;
  out ppNewFrame: PD3DXFrame): HResult;
var
  Frame: PD3DXFrameDerived;
begin
  New(Frame);
  Frame.UserData:= 0;
  ppNewFrame:= PD3DXFrame(Frame);
  // initialize other data members of the frame
  D3DXMatrixIdentity(ppNewFrame.TransformationMatrix);
//  D3DXMatrixIdentity(ppNewFrame.CombinedTransformationMatrix);

  ppNewFrame.pMeshContainer := nil;
  ppNewFrame.pFrameSibling := nil;
  ppNewFrame.pFrameFirstChild := nil;

  ppNewFrame.Name:= StrNew(Name);

  if RootFrame = nil then RootFrame:= ppNewFrame;
  Result:= S_Ok;
end;

function TAllocateHierarchy.CreateMeshContainer(Name: PAnsiChar;
  const pMeshData: TD3DXMeshData; pMaterials: PD3DXMaterial;
  pEffectInstances: PD3DXEffectInstance; NumMaterials: DWORD;
  pAdjacency: PDWORD; pSkinInfo: ID3DXSkinInfo;
  out ppNewMeshContainer: PD3DXMeshContainer): HResult;
begin
  New(ppNewMeshContainer);
  FillChar(ppNewMeshContainer^, SizeOf(ppNewMeshContainer), 0);

  ppNewMeshContainer.Name:= StrNew(Name);
  ppNewMeshContainer.MeshData:= pMeshData;
  ppNewMeshContainer.NumMaterials:= NumMaterials;

  GetMem(ppNewMeshContainer.pMaterials, SizeOf(TD3DXMaterial)*NumMaterials);
  CopyMemory(ppNewMeshContainer.pMaterials, pMaterials, SizeOf(TD3DXMaterial)*NumMaterials);

  ppNewMeshContainer.pSkinInfo:= pSkinInfo;

  if (pEffectInstances <> nil) then
  begin
    //todo: Process "pEffectInstances" if required
    Application.MessageBox('"pEffectInstances" is assigned,'#13+
                           'but instancing of them is not realized!', 'Warning!!!');
  end;

  //todo: Also, handle adjacency (if required)
  // ppNewMeshContainer.pAdjacency:= pAdjacency; // - this is wrong

  Result:= S_Ok;
end;

function TAllocateHierarchy.DestroyMeshContainer(
  pMeshContainerToFree: PD3DXMeshContainer): HResult;
begin
  StrDispose(pMeshContainerToFree.Name);
  Dispose(pMeshContainerToFree);
  Result:= S_Ok;
end;

function TAllocateHierarchy.DestroyFrame(
  pFrameToFree: PD3DXFrame): HResult;
var
  Frame: PD3DXFrameDerived;
begin
  StrDispose(pFrameToFree.Name);
  Frame:= PD3DXFrameDerived(pFrameToFree);
  Dispose(Frame);
  Result:= S_Ok;
end;


//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

const
(*
  EffectStringGUID: TGUID = '{D55B097E-BDB6-4c52-B03D-6051C89D0E42}';
//  pXofSave.CreateDataObject(EffectStringGUID, 'Global_User_String', nil, 12, PChar('check_check'#0), Obj);
*)
  EffectDWORDGUID:  TGUID = '{622c0ed0-956e-4da9-908a-2af94f3ce716}';


{ TD3DXLoadUserData }

function TD3DXLoadUserData.LoadTopLevelData(
  pXofChildData: IDirectXFileData): HResult;
var
  dataGUID: PGUID;
  name: array[0..255] of Char;
  nameLen: DWORD;
  size: DWORD;
  pdw: PDWORD;
  Strings: TStrings;
begin
  pXofChildData.GetType(dataGUID);
  if IsEqualGUID(EffectDWORDGUID, dataGUID^) then
  begin
    nameLen:= SizeOf(Name);
    pXofChildData.GetName(@name, nameLen);
    pXofChildData.GetData(nil, size, Pointer(pdw));

    if LoadGlobalPre
      then Strings:= HierarchyForm.GlobalPreValueListEditor.Strings
      else Strings:= HierarchyForm.GlobalPostValueListEditor.Strings;

    with Strings do
    begin
      BeginUpdate; // to avoid bug in ValEdit
      Add(Format('%s=%d', [Name, pdw^]));
      EndUpdate;
    end;
  end;
  Result:= S_Ok;
end;

function TD3DXLoadUserData.LoadFrameChildData(pFrame: PD3DXFrame;
  pXofChildData: IDirectXFileData): HResult;
var
  dataGUID: PGUID;
  size: DWORD;
  pdw: PDWORD;
  Frame: PD3DXFrameDerived;

  name: array[0..255] of Char;
  nameLen: DWORD;
begin
  pXofChildData.GetType(dataGUID);
  if IsEqualGUID(EffectDWORDGUID, dataGUID^) then
  begin
    nameLen:= SizeOf(Name);
    pXofChildData.GetName(@name, nameLen);

{   // This path really shouldn't be taken at load time -- BUT IT DOES - so it's a bug in D3DX9
    if name = 'AddMeshChildData_DWORD' then
    begin
      Application.MessageBox('very strange', 'ssstrange');
      Exit;
    end; }
    pXofChildData.GetData(nil, size, Pointer(pdw));

    Frame:= PD3DXFrameDerived(pFrame);
    Frame.UserData:= pdw^;
  end;
  LoadGlobalPre:= False;
  Result:= S_Ok;
end;

function TD3DXLoadUserData.LoadMeshChildData(
  pMeshContainer: PD3DXMeshContainer;
  pXofChildData: IDirectXFileData): HResult;
{var
  dataGUID: PGUID;
  name: array[0..255] of Char;
  nameLen: DWORD;
  size: DWORD;
  pdw: PDWORD; }
begin
//todo: Seems Loading/Saving MeshChildData is not working correctly in D3DX 9.0 release
(*
  pXofChildData.GetType(dataGUID);
  if IsEqualGUID(EffectDWORDGUID, dataGUID^) then
  begin
    nameLen:= SizeOf(Name);
    pXofChildData.GetName(@name, nameLen);
    pXofChildData.GetData(nil, size, Pointer(pdw));

    with HierarchyForm.MeshValueListEditor.Strings do
    begin
      BeginUpdate; // to avoid bug in ValEdit
      Add(Format('%s=%d', [Name, pdw^]));
      EndUpdate;
    end;
  end; *)
  Result:= S_Ok;
end;

{ TD3DXSaveUserData }

function TD3DXSaveUserData.AddFrameChildData(pFrame: PD3DXFrame;
  pXofSave: IDirectXFileSaveObject;
  pXofFrameData: IDirectXFileData): HResult;
var
  Obj: IDirectXFileData;
  dw: DWORD;
begin
  dw:= Counter;
  if not AutoGenerateUserData then dw:= PD3DXFrameDerived(pFrame).UserData;

  pXofSave.CreateDataObject(EffectDWORDGUID, 'AddFrameChildData_DWORD', nil, 4, @dw, Obj);
  pXofFrameData.AddDataObject(Obj);
  Result:= S_Ok;
  Inc(Counter);
end;

function TD3DXSaveUserData.AddMeshChildData(
  pMeshContainer: PD3DXMeshContainer; pXofSave: IDirectXFileSaveObject;
  pXofMeshData: IDirectXFileData): HResult;
{var
  Obj: IDirectXFileData;
  dw: DWORD;}
begin
{  dw:= Counter2;
  pXofSave.CreateDataObject(EffectDWORDGUID, 'AddMeshChildData_DWORD', nil, 4, @dw, Obj);
  pXofMeshData.AddDataObject(Obj); }
  Result:= S_Ok;
  Inc(Counter2)
end;

function TD3DXSaveUserData.AddTopLevelDataObjectsPre(
  pXofSave: IDirectXFileSaveObject): HResult;
var
  Obj: IDirectXFileData;
  dw: DWORD;
  i: Integer;
begin
  if HierarchyForm.GlobalPreValueListEditor.Strings.Count > 0 then
  with HierarchyForm.GlobalPreValueListEditor do
  begin
    for i:= 0 to Strings.Count - 1 do
    begin
      dw:= StrToIntDef(Strings.Values[Strings.Names[i]], 0);
      pXofSave.CreateDataObject(EffectDWORDGUID, PChar(Strings.Names[i]), nil, 4, @dw, Obj);
      pXofSave.SaveData(Obj);
    end;
  end else // Just add some predefined data
  begin
    dw:= 12345;
    pXofSave.CreateDataObject(EffectDWORDGUID, 'Global_User_DWORD_Pre', nil, 4, @dw, Obj);
    pXofSave.SaveData(Obj);
    dw:= 12021;
    pXofSave.CreateDataObject(EffectDWORDGUID, 'Global_User_DWORD_Pre2', nil, 4, @dw, Obj);
    pXofSave.SaveData(Obj);
  end;
  Result:= S_Ok;
end;

function TD3DXSaveUserData.AddTopLevelDataObjectsPost(
  pXofSave: IDirectXFileSaveObject): HResult;
var
  Obj: IDirectXFileData;
  dw: DWORD;
  i: Integer;
begin
  if HierarchyForm.GlobalPostValueListEditor.Strings.Count > 0 then
  with HierarchyForm.GlobalPostValueListEditor do
  begin
    for i:= 0 to Strings.Count - 1 do
    begin
      dw:= StrToIntDef(Strings.Values[Strings.Names[i]], 0);
      pXofSave.CreateDataObject(EffectDWORDGUID, PChar(Strings.Names[i]), nil, 4, @dw, Obj);
      pXofSave.SaveData(Obj);
    end;
  end else // Just add some predefined data
  begin
    dw:= 54321;
    pXofSave.CreateDataObject(EffectDWORDGUID, 'Global_User_DWORD_Post', nil, 4, @dw, Obj);
    pXofSave.SaveData(Obj);
  end;
  Result:= S_Ok;
end;

function TD3DXSaveUserData.RegisterTemplates(
  pXFileApi: IDirectXFile): HResult;
begin
  Result:= pXFileApi.RegisterTemplates(PChar(XSKINEXP_TEMPLATES), Length(XSKINEXP_TEMPLATES));
end;

function TD3DXSaveUserData.SaveTemplates(
  pXofSave: IDirectXFileSaveObject): HResult;
begin
  Result:= S_Ok;
end;

end.

